import serial
import modbus_tk.defines as cst
from modbus_tk import modbus_rtu
import threading
from module.myMqtt import myMqtt
import time
#############################[Sensor Config]###################################

#   485         Addr        Index
#   1           2           3
#   建大jd      上洛sl      兆泰盛zt
#   0x00        0x06        0x00

SerialCom = 'COM18'

# (sensor_id, register)
noise_Message = [   (1, 0x00),
                    (2, 0x06),
                    (3, 0x00)]

# Enable sensor
enableJianDa    = 1
enableShangLuo  = 0
enableZhaotai   = 1

# Enable Print OnLineTime
enableOnLineTime = 0

#############################[MQTT Config]###################################

RequestTopic = 'WT/WTG93RF/99991213/opDataRequest'
EchoNoiseTopic = 'WT/WTG93RF/99991213/opData'



begin_time = time.time()
class Sensor:
    def __init__(self, port):
        self._port = port
        try:
            self._ser = serial.Serial(self._port, baudrate=4800)
            self._ser.open() if not self._ser.is_open else None
            self.__master = modbus_rtu.RtuMaster(self._ser)
            self.__master.set_timeout(2, use_sw_timeout=True)
        except Exception as exp:
            print(exp) 

    def get_noise(self, noise_sensor_idx):
        noise_sensor_idx -= 1
        data = self.__master.execute(noise_Message[noise_sensor_idx][0], cst.READ_HOLDING_REGISTERS, noise_Message[noise_sensor_idx][1], 1)
        return data

def getNoise(sendData):
        ser.write(bytes(sendData))
        time.sleep(0.5) # 需要等待
        address = ser.read(1)  # 读取地址字节
        function_code = ser.read(1)  # 读取功能码字节
        data_length = ser.read(1)  # 读取数据字节数字节
        data = ser.read(2)
        crc = ser.read(2)  # 读取CRC校验（2字节）
        # 将字节串转换为整数
        dB_data = int.from_bytes(data, byteorder='big')
        dB_data = dB_data / 10.0  # trans dB
        return dB_data


S = Sensor(SerialCom)
# ser = serial.Serial(SerialComTest, 4800, parity=serial.PARITY_NONE, bytesize=serial.EIGHTBITS, stopbits=serial.STOPBITS_ONE)
# data_to_send = [0x02,0x03,0x00,0x06,0x00, 0x01 ,0x64 ,0x38]

def mqtt_task():
    try:
        while True:
            if mqtt._start_flag:
                if enableJianDa:
                    time.sleep(0.3)
                    n1 = S.get_noise(1)[0] / 10.0
                else:
                    n1 = 0.0
                
                if enableShangLuo:
                    time.sleep(0.3)
                    n2 = S.get_noise(2)[0] / 10.0
                else:
                    n2 = 0.0

                if enableZhaotai:
                    time.sleep(0.3)
                    n3 = S.get_noise(3)[0] / 10.0
                else:
                    n3 = 0.0

                now = int(time.time())
                print(time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(now)))
                data = [{"addr":"1","ts":int(now),"jd":float(n1),"sl":float(n2),"zt":float(n3)}]
                # data = json.dumps(data)
                mqtt.publish(EchoNoiseTopic, data)
                print(data)
                time.sleep(1)
                if enableOnLineTime:
                    end_time = time.time()
                    run_time = round(end_time-begin_time)
                    hour = run_time//3600
                    minute = (run_time-3600*hour)//60
                    second = run_time-3600*hour-60*minute
                    print(f"online-time:{hour}:{minute}:{second}")
    except Exception as exp:
        print("Error Exception:", exp)
        print("Need to restart the script")

if __name__ == '__main__':
    mqtt_info = {
        'broker': 'www.witium.com.cn',
        'port': 12883,
        'user': 'witcd',
        'password': 'Witium37774020'
    }
    mqtt = myMqtt(mqtt_info)
    mqtt.set_add_topic(RequestTopic)
    mqtt.open_mqtt()

    mqtt_thread = threading.Thread(target=mqtt.loop_task)
    mqtt_thread.start()

    app_thread = threading.Thread(target=mqtt_task)
    app_thread.start()
